import os, glob, csv

# THREE REQUIREMENTS
# - pdfcairo from poppler-utils
# - imagemagick

# WARNING <===========================================================
# - CSV should be first sorted by date (col 10), then by Topic (col 1)
# ===========================================================> WARNING


fname = "ConTeXt_Garden_manuals_database_20231202.csv"

os.makedirs("./images", exist_ok=True)
os.makedirs("./thumbs", exist_ok=True)

#=====================================================================
# Open CSV database of the manuals
#
# 1/ Build table of categories: Category
#    Count in each category:
#    - number of UpToDate manuals : CategoryNBa
#    - number of Old manuals : CategoryNBb
#
# 2/ Download PDF file in manual subfolder, if needed
#    Extract picture from 1st page with pdftocaire
#    Produce thumbs, with OBSOLETE banner if not uptodate (col 6 csv)
#
# 3/ Build Synthese Table containing only required element
#    - one manual succesfully processed = 1 row
#    - col0 : category from csv table, without number
#    - col1 : name of thumbs file, with extensions
#    - col2 : text from cvs table to present the manual + the date
#    - col3 : uptodate or not
#    - col4 : web url of the doc
#

file = open(fname, "r")
reader = csv.reader(file,delimiter = ';')

Synthese = []
Category = []
CategoryNBa=[]  #could be useful to play with css grid and span
CategoryNBb=[]  #could be useful to play with css grid and span
Ci=0

next(reader)
for row in reader:

    # Build category table and count nb of uptodate and old manual
    if row[0] not in Category:
        Category.append(row[0])
        if row[5] == "1":
            CategoryNBa.append(1)
            CategoryNBb.append(0)
        else:
            CategoryNBa.append(0)
            CategoryNBb.append(1)
        Ci=Ci+1
    else:
        # Assumption : database is sorted by categories
        if row[5] == "1":
            CategoryNBa[Ci-1]+=1
        else:
            CategoryNBb[Ci-1]+=1

    # Download PDF file in manual subfolder, if needed
    if not (os.path.isfile('manuals/'+row[3])):
        os.system("wget -P manuals "+row[2])

    # Produce thumbnails, if needed
    # first 2000px HD, to get details with pdfcairo, then reduce
    if not (os.path.isfile('thumbs/'+row[3][:-4]+".jpg")):
        os.system("pdftocairo manuals/"+row[3]+
                  " -singlefile -png -scale-to 2006")
        # Get dimensions W and H
        W1=os.system("identify "+row[3][:-4]+".png "+ 
                     "| cut -f 3 -d \" \" | sed s/x.*//")
        H1=os.system("identify "+row[3][:-4]+".png "+
                     "| cut -f 3 -d \" \" | sed s/.*x//")
        # Dirty tricks to reduce size
        W2=W1 - 6
        H2=H1 - 6
        os.system("convert "+row[3][:-4]+".png "+
                  "-crop "+str(W2)+"'x'"+str(H2)+"+3+3 "+ 
                  "-resize 300 -quality 90% " +
                  "thumbs/"+row[3][:-4]+".jpg ")
        os.system("mv "+row[3][:-4]+".png images")

        # Add obsolete banner if not uptodate doc
        if row[5] != "1":
#            os.system("mogrify -fill white -pointsize 40 " +
#                      "-undercolor  red "+
#                      "-gravity center -annotate +0+60 \'DEPRECATED\' thumbs/"+row[3][:-4]+".jpg")

            os.system("mogrify -gravity Center -background \'#FF000080\' -splice 0x60 "+
                      "-fill white -pointsize 42 " +
                      "-annotate +0+5 \'DEPRECATED\' thumbs/"+row[3][:-4]+".jpg")



    # If thumbnail exists (= success of previous steps)
    # then add the manual to the Synthese table for next steps

    if (os.path.isfile('thumbs/'+row[3][:-4]+".jpg")):
        Synthese.append([row[0][4:], (row[3][:-4])+".jpg", 
                         row[1] + " ("+row[9]+")", row[5],row[2]])

file.close()

#=====================================================================
# Start writing mediawiki syntaxe for a Manual Page

fouta = open("ConTeXt_Garden_wiki_MainPage_outputa.txt", "w")
foutb = open("ConTeXt_Garden_wiki_MainPage_outputb.txt", "w")

# thumbnails width (2 boxes in smartphone portrait, 3 in landscape)
ew="100px"

# Start with the category, open first ul (category) and li (title)
# Assumption : there is old and uptodate doc in the first category

countitem = 0
countcat = 0


fouta.write("<ul class=\"filter-links\">\n")
fouta.write("<li></li>\n")
for j in Synthese:
    if (j[0] != Synthese[countitem-1][0]) :
        countcat+=1
        fouta.write("<li>"+j[0].replace(" ","")+"</li>\n")
    countitem+=1
fouta.write("</ul>\n\n")
fouta.write("<ul class=\"filter-sections\">\n")

foutb.write("\n\n= Deprecated manuals =\n")
foutb.write("<ul class=\"filter-sections\">\n")

#======================================================

for j in Synthese:

    # Then if uptodate, write in fouta, else in foutb
    if j[3] == "1":
        fouta.write("<li class=\""+j[0].replace(" ","")+"\">")
        fouta.write("[[File:"+j[1]+"|"+ew+"|frameless|link="+j[4]+"]]"+
                    "<div style=\"padding-left:.5em; text-align:left;\">["+j[4]+" "+j[2]+"]</div>")
        fouta.write("</li>\n")
    else:
        foutb.write("<li class=\""+j[0].replace(" ","")+"\">")
        foutb.write("[[File:"+j[1]+"|"+ew+"|frameless|link="+j[4]+"]]"+
                    "<div style=\"padding-left:.5em; text-align:left;\">["+j[4]+" "+j[2]+"]</div>")
        foutb.write("</li>\n")

fouta.write("</ul>\n")
foutb.write("</ul>\n")

fouta.close()
foutb.close()

# Concatenate

fout = open("ConTeXt_Garden_wiki_MainPage_output.txt", "w")
fout.write(''.join([open(f).read() for f in \
                    ["ConTeXt_Garden_wiki_MainPage_outputa.txt" , 
                     "ConTeXt_Garden_wiki_MainPage_outputb.txt"]]))
fout.close()
